--
-- rotorSpreader Specialization (FS17)
--
-- 07/11/2019
-- FS19 Version v1.0.0.0
-- Converted by Mythos
-- www.ARM-team.gr
-- Copyright (C) ARM Team, All Rights Reserved.

RotorSpreader = {};

RotorSpreader.STATE_DISABLED = 0;
RotorSpreader.STATE_STARTING = 1;
RotorSpreader.STATE_RUNNING = 2;
RotorSpreader.STATE_STOPPING = 3;
RotorSpreader.STATE_STOP = 4;

function RotorSpreader.prerequisitesPresent(specializations)
	return true;
end;

--___________________________________________________________________________________________________
function RotorSpreader.registerEventListeners(vehicleType)
	
	SpecializationUtil.registerEventListener(vehicleType, "onPreLoad", RotorSpreader);
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", RotorSpreader);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", RotorSpreader);
	SpecializationUtil.registerEventListener(vehicleType, "onDeactivate", RotorSpreader);
	SpecializationUtil.registerEventListener(vehicleType, "onTurnedOn", RotorSpreader);
	SpecializationUtil.registerEventListener(vehicleType, "onTurnedOff", RotorSpreader);
end;

--___________________________________________________________________________________________________
function RotorSpreader:onPreLoad(savegame)
end;

--___________________________________________________________________________________________________
function RotorSpreader:onLoad(savegame)
	self.animation = {};
    self.animation.state = RotorSpreader.STATE_DISABLED;
    self.animation.startWeight = 1;
    self.animation.runWeight = 0;
    self.animation.endWeight = 0;
    self.animation.crossFadeTime = 200;
    self.animation.runThreshold = 0.9;
    self.animation.rootNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.rotorAnimation#rootNode"), self.i3dMappings);
    if self.animation.rootNode ~= nil then
        self.animation.startClipSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.rotorAnimation#startClipSpeed", 1));
        self.animation.runClipSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.rotorAnimation#runClipSpeed", 1));
        self.animation.endClipSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.rotorAnimation#endClipSpeed", 1));
        self.animation.animCharSet = getAnimCharacterSet(self.animation.rootNode);
        if self.animation.animCharSet ~= 0 then
            local startClip = getAnimClipIndex(self.animation.animCharSet, getXMLString(self.xmlFile, "vehicle.rotorAnimation#startClip"));
            local runClip = getAnimClipIndex(self.animation.animCharSet, getXMLString(self.xmlFile, "vehicle.rotorAnimation#runClip"));
            local endClip = getAnimClipIndex(self.animation.animCharSet, getXMLString(self.xmlFile, "vehicle.rotorAnimation#endClip"));
            assignAnimTrackClip(self.animation.animCharSet, 0, startClip);
            assignAnimTrackClip(self.animation.animCharSet, 1, runClip);
            assignAnimTrackClip(self.animation.animCharSet, 2, endClip);
            self.animation.startDuration = getAnimClipDuration(self.animation.animCharSet, startClip);
            self.animation.runDuration = getAnimClipDuration(self.animation.animCharSet, runClip);
            self.animation.endDuration = getAnimClipDuration(self.animation.animCharSet, endClip);
            setAnimTrackSpeedScale(self.animation.animCharSet, 0, self.animation.startClipSpeed);
            setAnimTrackSpeedScale(self.animation.animCharSet, 1, self.animation.runClipSpeed);
            setAnimTrackSpeedScale(self.animation.animCharSet, 2, self.animation.endClipSpeed);
            setAnimTrackLoopState(self.animation.animCharSet, 0, false);
            setAnimTrackLoopState(self.animation.animCharSet, 1, true);
            setAnimTrackLoopState(self.animation.animCharSet, 2, false);
            setAnimTrackTime(self.animation.animCharSet, 0, 0);
            setAnimTrackTime(self.animation.animCharSet, 1, 0);
            setAnimTrackTime(self.animation.animCharSet, 2, 0);
            setAnimTrackBlendWeight(self.animation.animCharSet, 0, self.animation.startWeight);
            setAnimTrackBlendWeight(self.animation.animCharSet, 1, self.animation.runWeight);
            setAnimTrackBlendWeight(self.animation.animCharSet, 2, self.animation.endWeight);
        end;
        self.initializeAnimation = true;
        self.isAnimationRunning = false;
    end
	
	--local spec = self.spec_RotorSpreader;
end;

--___________________________________________________________________________________________________
function RotorSpreader:onUpdate(dt)
    if self.animation.rootNode == nil or self.animation.animCharSet == 0 then
        return;
    end
    if self.initializeAnimation then
        enableAnimTrack(self.animation.animCharSet, 0);
        setAnimTrackTime(self.animation.animCharSet, 0, 0, true);
        disableAnimTrack(self.animation.animCharSet, 0);
        self.initializeAnimation = false;
    end;
    if self.animation.state ~= RotorSpreader.STATE_DISABLED then
        if self.animation.state == RotorSpreader.STATE_STARTING then
            self.animation.startWeight = math.min(self.animation.startWeight + dt/self.animation.crossFadeTime, 1);
            self.animation.endWeight = 1-self.animation.startWeight;
            if getAnimTrackTime(self.animation.animCharSet, 0) >= self.animation.startDuration then
                self.animation.runWeight = 1;
                self.animation.startWeight = 0;
                self.animation.endWeight = 0;
                setAnimTrackTime(self.animation.animCharSet, 0, 0);
                setAnimTrackTime(self.animation.animCharSet, 1, 0);
                enableAnimTrack(self.animation.animCharSet, 1);
                disableAnimTrack(self.animation.animCharSet, 0);
                self.animation.state = RotorSpreader.STATE_RUNNING;
            end;
        elseif self.animation.state == RotorSpreader.STATE_STOP then
            local runTime = getAnimTrackTime(self.animation.animCharSet, 1);
            if runTime >= self.animation.runDuration*self.animation.runThreshold then
                setAnimTrackTime(self.animation.animCharSet, 2, 0);
                enableAnimTrack(self.animation.animCharSet, 2);
                self.animation.state = RotorSpreader.STATE_STOPPING
            end;
        elseif self.animation.state == RotorSpreader.STATE_STOPPING then
            self.animation.endWeight = math.min(self.animation.endWeight + dt/self.animation.crossFadeTime, 1);
            self.animation.runWeight = 1-self.animation.endWeight;
            if getAnimTrackTime(self.animation.animCharSet, 2) >= self.animation.endDuration then
                disableAnimTrack(self.animation.animCharSet, 2);
                disableAnimTrack(self.animation.animCharSet, 1);
                self.animation.state = RotorSpreader.STATE_DISABLED;
                self.animation.runWeight = 0;
            end;
        end;
        setAnimTrackBlendWeight(self.animation.animCharSet, 0, self.animation.startWeight);
        setAnimTrackBlendWeight(self.animation.animCharSet, 1, self.animation.runWeight);
        setAnimTrackBlendWeight(self.animation.animCharSet, 2, self.animation.endWeight);
    end;

end;

--___________________________________________________________________________________________________
function RotorSpreader:onDeactivate()
    if self.animation.rootNode == nil or self.animation.animCharSet == 0 then
        return;
    end
    if self.animation.state == RotorSpreader.STATE_RUNNING then
        self.animation.state = RotorSpreader.STATE_STOP;
    end;
    if self.animation.state == RotorSpreader.STATE_STARTING then
        self.animation.state = RotorSpreader.STATE_STOPPING;
    end;
end;

--___________________________________________________________________________________________________
function RotorSpreader:onTurnedOn(noEventSend)
    if self.animation.rootNode == nil or self.animation.animCharSet == 0 then
        return;
    end
    setAnimTrackTime(self.animation.animCharSet, 0, 0);
    enableAnimTrack(self.animation.animCharSet, 0);
    self.animation.state = RotorSpreader.STATE_STARTING;
end;

--___________________________________________________________________________________________________
function RotorSpreader:onTurnedOff(noEventSend)
    if self.animation.rootNode == nil or self.animation.animCharSet == 0 then
        return;
    end
    self.animation.state = RotorSpreader.STATE_STOP;
end;